#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
#   Copyright (C) 2005 Adrian Likins <adrian@likins.com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#  Renders each glyph of a font into a brush image hose. Handy for say, creating alot
#  of brushes from a dingbat font.
#
# TODO:
#      Would be cool to be able to query the font info and find out which glyphs are in the font
#      since a lot of dingbats are incomplete. Maybe use pygtk->pango/etc?


# MODIFIED VERSION FOR WINDOWS by Mahvin <mahvin@gmail.com> August 25, 2009 http://registry.gimp.org/node/18273

# MODIFIED VERSION FOR WINDOWS by MareroQ 25-01-2014:

1. File in Windows does not distinguish the size of letters ( "a.svg" = "A.svg" )- it to avoid overwriting - added "space" for upper case ( "a.svg" no = "A .svg" ).
2. It is a combination of two plug-ins Adrian Likins ( http://www.adrianlikins.com/misc/gimp/ ): font-to-brushes.py and font-to-image-hose.py
3. This plug-ins calls two scripts by Saul Goode [special thanks] http://gimpchat.com/viewtopic.php?f=9&t=9511
4. Add decode("utf-8") as text-brush.py Author: João Sebastião de Oliveira Bueno [gwidion]
5. Add fontname to layers as anitools.py Author: Lars (lars.stonerocket.co.uk)

The installation: Font-to-svg-and-brushes.py to directory plug-ins, Sg-font-to-svg-files.scm to directory scripts.
Attention: Do not use these characters \/?:*.><| to save the file *.svg
'''

import os, re
import string

import gimp
from gimpfu import *
import pygtk
pygtk.require("2.0")
import gtk


def messBox(message, type, modal):
    if modal == 0:
      flag = gtk.DIALOG_DESTROY_WITH_PARENT
    else:
      flag = gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT
    msgBox = gtk.MessageDialog(None, flag, type, gtk.BUTTONS_OK, message)
    ret = msgBox.run()
    msgBox.destroy()

def get_gimp_version():
    # assume version like "gimp-2.8"
    ver = pdb.gimp_version()
    parts = ver.split(".")
    #    print parts
    major_minor_ver = string.join(parts[0:2], ".")
    return major_minor_ver
    
def add_name_to_all_layers(img, name): 
	if name:
		for layer in img.layers:
			layer.name += name
	else:
		number = 0
		for layer in img.layers:
			layer.name = str(number)
			number += 1
			
def font_to_image_hose_small(font, chars, size, selectionmode):
    print font, chars, size, selectionmode
    chars = chars.decode("utf-8")
    # ugh, the font selector widget seems to want to return a PDB_STRING of format "fontname fontsize"
    #  so, we just strip that bit off...
    parts = font.split(" ")
    if parts[-1].isdigit():
        fontparts  = parts[:-1]
        font = string.join(fontparts, " ")

#   brushdir = "%s/.gimp-%s/brushes/" % (os.path.expandvars("$HOME"), get_gimp_version())    <- for Linux    
    brushdir = "%s/.gimp-%s/brushes/" % (os.path.expanduser('~'), get_gimp_version())		 # MODIFIED VERSION FOR WINDOWS by Mahvin 
    max_width = 0.0
    max_height = 0.0
    for char in chars:
        (width, height, asc, dsc) = pdb.gimp_text_get_extents_fontname(char, size, 1, font)
        if width > max_width:
            max_width = width
        if height > max_height:
            max_height = height

    img = gimp.Image(max_width, max_height, RGB)

    layer_count = 0
    for char in chars:
        (width, height, asc, dsc) = pdb.gimp_text_get_extents_fontname(char, size, 1, font)

        drawable = gimp.Layer(img, "%s " % char, width, height, RGBA_IMAGE, 100, NORMAL_MODE)
        img.add_layer(drawable, layer_count)
        layer_count = layer_count + 1
        pdb.gimp_edit_clear(drawable)
        font_layer = pdb.gimp_text_fontname(img, drawable,  0.0, 0.0, char, 0, 1, size, 1, font)

        pdb.gimp_floating_sel_anchor(font_layer)

    name = "%s_%s"  % (font,size)
    filename = "%s/%s.gih" % (brushdir,name)

    print img, drawable, filename, filename, 50, name, max_width, max_height, 1,1, 1, [len(chars)], 1, [selectionmode]
    pdb.file_gih_save(img, drawable, filename, filename, 50, name, max_width, max_height, 1,1,1, [len(chars)], 1, [selectionmode])
    disp = gimp.Display(img)
    pdb.python_fu_add_name_to_all_layers( img, name, run_mode=RUN_INTERACTIVE)
    pdb.sg_layer_to_path( img, drawable, 1, run_mode=RUN_INTERACTIVE)
    pdb.sg_alpha_to_all_svg_files( img, drawable, 1, 1, run_mode=RUN_INTERACTIVE)
    messBox("""PLUG-INS FONT-TO-SVG FINISHED WORKING
    .....................................................
    If you want to saved  all path to one file *.svg use:
    Path [context menu] ->Export Path.
    In windows [ Export Path to SVG ] chose:
    Export all paths from this image
    .....................................................
    Author plug-ins: Adrian Likins
    Author scripts: Saul Goode [special thanks]
    Modification and connection: MareroQ""", gtk.MESSAGE_INFO, 1)    
    return
	

def font_to_image_hose_big(font, chars, size, selectionmode):
    print font, chars, size, selectionmode
    chars = chars.decode("utf-8")
    parts = font.split(" ")
    if parts[-1].isdigit():
        fontparts  = parts[:-1]
        font = string.join(fontparts, " ")

#   brushdir = "%s/.gimp-%s/brushes/" % (os.path.expandvars("$HOME"), get_gimp_version())    <- for Linux
    brushdir = "%s/.gimp-%s/brushes/" % (os.path.expanduser('~'), get_gimp_version())        # MODIFIED VERSION FOR WINDOWS by Mahvin 
    max_width = 0.0
    max_height = 0.0
    for char in chars:
        (width, height, asc, dsc) = pdb.gimp_text_get_extents_fontname(char, size, 1, font)
        if width > max_width:
            max_width = width
        if height > max_height:
            max_height = height

    img = gimp.Image(max_width, max_height, RGB)

    layer_count = 0
    for char in chars:
        (width, height, asc, dsc) = pdb.gimp_text_get_extents_fontname(char, size, 1, font)

        drawable = gimp.Layer(img, "%s  " % char, width, height, RGBA_IMAGE, 100, NORMAL_MODE)
        img.add_layer(drawable, layer_count)
        layer_count = layer_count + 1
        pdb.gimp_edit_clear(drawable)
        font_layer = pdb.gimp_text_fontname(img, drawable,  0.0, 0.0, char, 0, 1, size, 1, font)

        pdb.gimp_floating_sel_anchor(font_layer)

    name = "%s_%s"  % (font, size)
    filename = "%s/%s_big.gih" % (brushdir,name)
    print img, drawable, filename, filename, 50, name, max_width, max_height, 1,1, 1, [len(chars)], 1, [selectionmode]
    pdb.file_gih_save(img, drawable, filename, filename, 50, name, max_width, max_height, 1,1,1, [len(chars)], 1, [selectionmode])
    disp = gimp.Display(img)
    pdb.python_fu_add_name_to_all_layers( img, name, run_mode=RUN_INTERACTIVE)
    pdb.sg_layer_to_path( img, drawable, 1, run_mode=RUN_INTERACTIVE)
    pdb.sg_alpha_to_all_svg_files( img, drawable, 1, 1, run_mode=RUN_INTERACTIVE)
    messBox("""PLUG-INS FONT-TO-SVG FINISHED WORKING
    .....................................................
    If you want to saved  all path to one file *.svg use:
    Path [context menu] ->Export Path.
    In windows [ Export Path to SVG ] chose:
    Export all paths from this image
    .....................................................
    Author plug-ins: Adrian Likins
    Author scripts: Saul Goode [special thanks]
    Modification and connection: MareroQ""", gtk.MESSAGE_INFO, 1)    
    return
	
register(
    "python_fu_font_to_image_hose_small",
    "Convert a font (Lower Case, Numerals, Characters) to file *.svg and brush *.gih (autosave to directory brush)",
    "Convert a font to an image hose brush",
    "Adrian Likins",
    "Adrian Likins",
    "2005",
    "<Toolbox>/Filters/Text/Font to svg and brush gih (Lower Case, Numerals, Characters)",
    "",
    [
    (PF_FONT, "font", "Font to convert ", "Sans"),
    (PF_STRING, "text", "Characters", "~!@#$%^&()_+-=;',{}[]abcdefghijklmnopqrstuvxyz0123456789"),
    (PF_FLOAT, "size", "Size of Font", 20.0),
    (PF_RADIO, "selectionmode",  "Method of the brush:",
     "incremental", (("incremental", "incremental"),
                     ("random", "random"),
                     ("angular", "angular"),
                     ("velocity", "velocity"),
                     ("pressure", "pressure"),
                     ("xtilt", "xtilt"),
                     ("ytilt", "ytilt"))),
    ],
    [],
    font_to_image_hose_small)
	
register(
    "python_fu_font_to_image_hose_big",
     "Convert a font (Upper Case) to file *.svg and brush *.gih (autosave to directory brush)",
    "Convert a font to an image hose brush",
    "Adrian Likins",
    "Adrian Likins",
    "2005",
    "<Toolbox>/Filters/Text/Font to svg and brushe gih (Upper Case)...",
    "",
    [
    (PF_FONT, "font", "Font to convert ", "Sans"),
    (PF_STRING, "glyphs", "Characters", "ABCDEFGHIJKLMNOPQRSTUVXYZ"),
    (PF_FLOAT, "size", "Size of Font", 20.0),
    (PF_RADIO, "selectionmode",  "Method of the brush: ",
     "incremental", (("incremental", "incremental"),
                     ("random", "random"),
                     ("angular", "angular"),
                     ("velocity", "velocity"),
                     ("pressure", "pressure"),
                     ("xtilt", "xtilt"),
                     ("ytilt", "ytilt"))),
    ],
    [],
    font_to_image_hose_big)
	
register(
    proc_name=("python-fu-add-name-to-all-layers"),
    blurb=(""),
    help=("Help?"),
    author=(""),
    copyright=("GPL"),
    date=("2014"),
    label=(""),
    imagetypes=("*"),
    params=[
	(PF_IMAGE, "img", "Image", None),
	(PF_STRING, "name", "Add text:", ""),
	],
    results=[],
    function=(add_name_to_all_layers), 
    menu=("")
 )
 

def font_to_brushes_small(font, chars, size):
    chars = chars.decode("utf-8")
    parts = font.split(" ")
    if parts[-1].isdigit():
        fontparts  = parts[:-1]
        font = string.join(fontparts, " ")
		
#   brushdir = "%s/.gimp-%s/brushes/" % (os.path.expandvars("$HOME"), get_gimp_version())    <- for Linux
    brushdir = "%s/.gimp-%s/brushes" % (os.path.expanduser('~'), get_gimp_version())		 # MODIFIED VERSION FOR WINDOWS by Mahvin 
    for char in chars:
        (width, height, asc, dsc) = pdb.gimp_text_get_extents_fontname(char, size, 1, font)
        img = gimp.Image(width, height, RGB)

        drawable = gimp.Layer(img, "test", width, height, RGB_IMAGE, 100, NORMAL_MODE)
        img.add_layer(drawable, 0)
        pdb.gimp_edit_fill(drawable, BACKGROUND_FILL)
        font_layer = pdb.gimp_text_fontname(img, drawable,  0.0, 0.0, char, 0, 1, size, 1, font)

        pdb.gimp_floating_sel_anchor(font_layer)

        pdb.plug_in_autocrop(img, drawable)
        pdb.gimp_image_convert_grayscale(img)

        name = "%s_%s_%s"  % (font, char, size)
        filename = "%s/%s.gbr" % (brushdir,name)
        pdb.file_gbr_save(img, drawable, filename, filename, 20, name)

        pdb.gimp_image_delete(img)

    pdb.gimp_brushes_refresh()
        
    return

register(
    "python_fu_font_to_brushes_small",
    "Convert a font to brushes *.gbr(Lower Case, Numerals, Characters)",
    "Convert a font to brushes",
    "Adrian Likins",
    "Adrian Likins",
    "2005",
    "<Toolbox>/Filters/Text/Font to brushes gbr (Lower Case, Numerals, Characters)...",
    "",
    [
    (PF_FONT, "font", "Font to convert to brushes", "Sans"),
    (PF_STRING, "glyphs", "Characters", "~!@#$%^&()_+-=;',{}[]abcdefghijklmnopqrstuvxyz0123456789"),
    (PF_FLOAT, "size", "Size of Font", 20.0),
    ],
    [],
    font_to_brushes_small)

def font_to_brushes_big(font, chars, size):
    chars = chars.decode("utf-8")
    parts = font.split(" ")
    if parts[-1].isdigit():
        fontparts  = parts[:-1]
        font = string.join(fontparts, " ")
#   brushdir = "%s/.gimp-%s/brushes/" % (os.path.expandvars("$HOME"), get_gimp_version())    <- for Linux
    brushdir = "%s/.gimp-%s/brushes" % (os.path.expanduser('~'), get_gimp_version())		 # MODIFIED VERSION FOR WINDOWS by Mahvin 
    for char in chars:
        (width, height, asc, dsc) = pdb.gimp_text_get_extents_fontname(char, size, 1, font)
        img = gimp.Image(width, height, RGB)

        drawable = gimp.Layer(img, "test", width, height, RGB_IMAGE, 100, NORMAL_MODE)
        img.add_layer(drawable, 0)
        pdb.gimp_edit_fill(drawable, BACKGROUND_FILL)
        font_layer = pdb.gimp_text_fontname(img, drawable,  0.0, 0.0, char, 0, 1, size, 1, font)

        pdb.gimp_floating_sel_anchor(font_layer)

        pdb.plug_in_autocrop(img, drawable)
        pdb.gimp_image_convert_grayscale(img)

        name = "%s_%s_%s"  % (font, char, size)
        filename = "%s/%s .gbr" % (brushdir,name)
        pdb.file_gbr_save(img, drawable, filename, filename, 20, name)

        pdb.gimp_image_delete(img)

    pdb.gimp_brushes_refresh()
        
    return

register(
    "python_fu_font_to_brushes_big",
    "Convert a font to brushes *.gbr (Upper Case)",
    "Convert a font to brushes",
    "Adrian Likins",
    "Adrian Likins",
    "2005",
    "<Toolbox>/Filters/Text/Font to brushes gbr (Upper Case)...",
    "",
    [
    (PF_FONT, "font", "Font to convert to brushes", "Sans"),
    (PF_STRING, "glyphs", "Characters", "ABCDEFGHIJKLMNOPQRSTUVXYZ"),
    (PF_FLOAT, "size", "Size of Font", 20.0),
    ],
    [],
    font_to_brushes_big)

main()


